/* -*- C -*- */
/*
 * Modified by : Pierre Schnizer January 2003
 *
 * Changelog: 
 *   22. May 2003. Changed to use the pygsl library. Warning! Do not import
 * Numeric/arrayobject.h before pygsl_block_helpers.h.  pygsl_block_helpers.h
 * defines the PY_ARRAY_UNIQUE_SYMBOL.
 */		 

%module bspline
%include gsl_error_typemap.i
%{
#include <pygsl/block_helpers.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_bspline.h>
#include <stdlib.h>
#include <stdio.h>
struct pygsl_bspline
{
     gsl_bspline_workspace *w;
};

%}

%rename(pygsl_bspline) bspline;

%feature("autodoc");

struct pygsl_bspline
{
     %immutable;     
     gsl_bspline_workspace *w;
};

%feature("autodoc") pygsl_bspline "XXX missing" "XXX yes! missing";
%extend pygsl_bspline{
  pygsl_bspline(size_t K, size_t NBREAK) {
       struct pygsl_bspline *tmp;
       tmp = calloc(1, sizeof(struct pygsl_bspline));
       if(tmp == NULL){
	    return NULL;
       }	    
       tmp->w = gsl_bspline_alloc(K, NBREAK);
       return tmp;
  }
  ~pygsl_bspline() {
       gsl_bspline_free(self->w);
       self->w = NULL;
       free(self);
  }

  gsl_error_flag_drop knots(const gsl_vector * BREAKPTS){
       return gsl_bspline_knots(BREAKPTS, self->w);
  }

  gsl_error_flag_drop knots_uniform(const double a, const double b){
       return gsl_bspline_knots_uniform(a, b, self->w);
  }
  PyObject*  eval(const double X){
       PyArrayObject *B_a = NULL;
       gsl_vector_view B_v;
       int flag=GSL_EFAILED;
       PyGSL_array_index_t n;

       n = self->w->n;
       B_a = PyGSL_New_Array(1, &n, PyArray_DOUBLE);
       if(B_a == NULL)
	    return NULL;
       B_v = gsl_vector_view_array((double *) (B_a->data), (B_a->dimensions[0]));
       flag = gsl_bspline_eval(X, &(B_v.vector), self->w);
       if (PyGSL_ERROR_FLAG(flag) != GSL_SUCCESS)
	    goto fail;
       return (PyObject *) B_a;
  fail:
       /* Failed */
       Py_XDECREF(B_a);
       return NULL;
  }
};

%init{
     init_pygsl();
} 
